/* This example shows how to get single-shot range
 measurements from the VL53L0X. The sensor can optionally be
 configured with different ranging profiles, as described in
 the VL53L0X API user manual, to get better performance for
 a certain application. This code is based on the four
 "SingleRanging" examples in the VL53L0X API.

 The range readings are in units of mm. */

#include <Wire.h>
#include "VL53L0X.h"

VL53L0X bunkerLevel;


// Uncomment this line to use long range mode. This
// increases the sensitivity of the sensor and extends its
// potential range, but increases the likelihood of getting
// an inaccurate reading because of reflections from objects
// other than the intended target. It works best in dark
// conditions.

//#define LONG_RANGE


// Uncomment ONE of these two lines to get
// - higher speed at the cost of lower accuracy OR
// - higher accuracy at the cost of lower speed

#define HIGH_SPEED
//#define HIGH_ACCURACY

int linearInterpolieren(int wertIn) {
  int arrayX[8][2] = { { 0,  100 },
                       { 40, 100 },   //0...40mm: voll, 100%
                       { 500, 40 },
                       { 600, 30 },
                       { 700, 20 },
                       { 800, 10 },
                       { 900,  0 },   //über 900: leer, 0%
                       { 8192, 0 } };

  int wertOut;  //Ausgangswert
  byte i = 0;
  while (i < 8 && wertIn > arrayX[i][0]) { i++; }  //passenden Eintrag im Array suchen
  wertOut = (((arrayX[i][1] - arrayX[i - 1][1]) * (wertIn - arrayX[i - 1][0])) / (arrayX[i][0] - arrayX[i - 1][0])) + arrayX[i - 1][1];
  return wertOut;
}

int bunkerFuellhoeheAuslesen() {
  int sensorWert = bunkerLevel.readRangeSingleMillimeters();
  if (bunkerLevel.timeoutOccurred()) {
    sensorWert = -1;
  } else {
    sensorWert = linearInterpolieren(sensorWert);
  }
  return sensorWert;
}

void setup() {
  Serial.begin(9600);
  Wire.begin();

  bunkerLevel.init();
  bunkerLevel.setTimeout(500);

#if defined LONG_RANGE
  // lower the return signal rate limit (default is 0.25 MCPS)
  bunkerLevel.setSignalRateLimit(0.1);
  // increase laser pulse periods (defaults are 14 and 10 PCLKs)
  bunkerLevel.setVcselPulsePeriod(VL53L0X::VcselPeriodPreRange, 18);
  bunkerLevel.setVcselPulsePeriod(VL53L0X::VcselPeriodFinalRange, 14);
#endif

#if defined HIGH_SPEED
  // reduce timing budget to 20 ms (default is about 33 ms)
  bunkerLevel.setMeasurementTimingBudget(20000);
#elif defined HIGH_ACCURACY
  // increase timing budget to 200 ms
  bunkerLevel.setMeasurementTimingBudget(200000);
#endif
}

void loop() {
  Serial.print("Fuellhoehe [%]: "); Serial.println(bunkerFuellhoeheAuslesen());
  //Serial.print("Letzte I2C-Transmission gab zurueck: ");
  //Serial.println(last_status, HEX);
  /* 0: success.
    1: data too long to fit in transmit buffer.
    2: received NACK on transmit of address.
    3: received NACK on transmit of data.
    4: other error.
    5: timeout */
  delay(100);
}
